#include "CycleReduction.h"

int CycleReductionMethod(double *x,
                         double a_0, double a_1, double a_2, 
                         double *a, double *b, double *c,
                         double *f, int n, int q)
{
    int i, j, k, start, elementsNum, step;
    double alpha, beta, x0, xn;
    if (a == NULL || b == NULL || c == NULL || f == NULL)
    {
        return OPERATION_FAILED;
    }
    a[0] = a_0;
    b[0] = a_1;
    c[0] = a_2;
    f[0] = 0;
    f[n] = 0;
    x0 = x[0];
    xn = x[n];
    x[0] = 0;
    x[n] = 0;
    start = 2;
    elementsNum = n;
    step = 1;
    for (j = 0; j < q - 1; j++)
    {
        alpha = -a[j] / b[j];
        beta  = -c[j] / b[j];
        a[j + 1] = alpha * a[j];
        b[j + 1] = b[j] + 2 * alpha * c[j];
        c[j + 1] = beta * c[j];
        elementsNum = (elementsNum - 1) / 2;
        #pragma omp parallel for num_threads(THREADS_NUMBER)
        for (i = 0; i < elementsNum; i++)
        {
            int k = start * (i + 1);
            f[k] = alpha * f[k - step] + f[k] + beta * f[k + step];
        }
        start *= 2;
        step  *= 2;
    }
    start = n / 2;
    step  = start;
    elementsNum = 1;
    for (j = q - 1; j >= 0; j--)
    {
        alpha = -a[j] / b[j];
        beta  = -c[j] / b[j];
        #pragma omp parallel for num_threads(THREADS_NUMBER)
        for (i = 0; i < elementsNum; i++)
        {
            int k = start * (2 * i + 1);
            x[k] = f[k] / b[j] + alpha * x[k - step] + beta * x[k + step];
        }
        start /= 2;
        step = start;
        elementsNum *= 2;
    }
    x[0] = x0;
    x[n] = xn;
    return OPERATION_OK;
}