#ifndef _DIFFEQUATION_H
#define _DIFFEQUATION_H

#include "Errors.h"
#include "CycleReduction.h"

#include <math.h>

#define MINPRICE 0.01
#define MAXPRICE 10000.0
#define EPS 0.000001

int computecoeff_a(double sigma, double delta_t, double delta_z, 
                   double D, double rf,
                   double *a_0, double *a_1, double *a_2);

int computecoeff_b(double sigma, double delta_t, double delta_z, 
                   double D, double rf,
                   double *b_0, double *b_1, double *b_2);

int matrvecmulti(double b_0, double b_1, double b_2,
                 double *vec, int J, double *res);

// in case k != n - 1
int rightboundcond(double K, double Ic,
                   double zmin, double delta_z, double rk,
                   double *ck0j, int J, double *res);

// in case k = n - 1
int rightboundcondlast(double K, double Ic,
                       double zmin, double delta_z, double rn,
                       int J, double *res);

double* r(double step, double delta_t, double rk_s, int n , int m);

int bottomboundcond(int kind, int m, int n, double step,
                    double K, double rf, double *rfunc, 
                    double delta_t, double *cond);

int topboundcond(int m, double K, double D, double Ic,
                 double delta_t, double step, double zmax, double *cond);


int crancknikolson(double T, double m, double sigma, double D, 
                   double K, double rk_s, double Ic, double rf,
                   int M, int J, int n, double zmin, double zmax,
                   double **cb_price, double *V);

// in case kind != n
int getoptimalstockprice(double K, double Ic, double zmin, double step, double *rfunc,
                         double delta_z, double delta_t, int J, int kind, 
                         double *c, double *V, int *optindex);

int getprices(double T, double m, double sigma, double D, 
              double K, double rk_s, double Ic, double rf,
              int M, int J, int n, 
              double **stockprice, double **cbprice);

#endif