#include "tbb.h"

class FFCompFunctor
{
private:
    double *f;
    double alpha;
    double beta;
    int step;
    int start;
public:
    FFCompFunctor(double *_f, 
                  const double _alpha,
                  const double _beta,
                  const int _step,
                  const int _start)
    {
        f = _f;
        alpha = _alpha;
        beta = _beta;
        step = _step;
        start = _start;
    }

    void operator()(const tbb::blocked_range<int> &r) const
    {
        int i, k;
        for (i = r.begin(); i < r.end(); i++)
        {
            k = start * (i + 1);
            f[k] = alpha * f[k - step] + f[k] + beta * f[k + step];
        }
    }
};

void fcomp(double *f, int elementsNum, double alpha, double beta, 
           int step, int start)
{
    tbb::parallel_for<tbb::blocked_range<int>, FFCompFunctor>(
                      tbb::blocked_range<int>(0, elementsNum), 
                      FFCompFunctor(f, alpha, beta, step, start), 
                      tbb::affinity_partitioner());
}

class FFCompreverseFunctor
{
private:
    double *x;
    double *f;
    double bj;
    double alpha;
    double beta;
    int step;
    int start;
public:
    FFCompreverseFunctor(double *_x, double *_f,
                         const double _bj, const double _alpha,
                         const double _beta, const int _step,
                         const int _start)
    {
        x = _x;
        f = _f;
        bj = _bj;
        alpha = _alpha;
        beta = _beta;
        step = _step;
        start = _start;
    }
    void operator()(tbb::blocked_range<int> &r) const
    {
        int i, k;
        for (i = r.begin(); i < r.end(); i++)
        {
            k = start * (2 * i + 1);
            x[k] = f[k] / bj + alpha * x[k - step] + beta * x[k + step];
        }
    }
};

void fcompreverse(double *x, int elementsNum, double *f, double bj, double alpha, 
                  double beta, int step, int start)
{
    tbb::parallel_for<tbb::blocked_range<int>, FFCompreverseFunctor>(
                      tbb::blocked_range<int>(0, elementsNum),
                      FFCompreverseFunctor(x, f, bj, alpha, beta, step, start),
                      tbb::affinity_partitioner());
}